#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, clean_var
from os import environ


def create_fs():
    print("INFO: Removing snapshot from FS.")
    errors = []

    snap_var_name = "LSM_" + environ["fmf_fs_snap_id"]
    fs_snap_id = read_var(snap_var_name)

    fs_id = read_var("LSM_" + environ["fmf_fs_id"])

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    atomic_run("Removing snapshot %s from FS %s" % (fs_snap_id, fs_id),
               command=lsm.fs_snap_delete,
               snap=fs_snap_id,
               fs=fs_id,
               force=True,
               errors=errors)

    atomic_run("Removing var %s" % snap_var_name,
               command=clean_var,
               var=snap_var_name,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_fs()
    exit(parse_ret(errs))



"""beaker.py: Module to manage beaker."""
from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."

import re  # regex
import os
import stqe.host.restraint
from libsan.host.cmdline import run


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def get_task_timeout(task_id):
    """
    Get how much time the task still has
    Parameter:
    \ttask_id:          Beaker Task ID
    Return:
    \tNone:             In case of some problem
    or
    \tint(value):       The remaining time in seconds
    """
    if not stqe.host.restraint.is_restraint_job():
        return None

    if task_id is None:
        _print("FAIL: beaker get_task_timeout() - requires task_id as parameter")
        return None

    cmd = "bkr watchdog-show %s" % task_id
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: beaker get_task_timeout() - Could not get beaker kill time")
        print(output)
        return None

    output_regex = re.compile(r"%s: (\d+)" % task_id)
    m = output_regex.match(output)
    if m:
        return int(m.group(1))
    _print("FAIL: beaker get_task_timeout() - Could not parse output:")
    print(output)
    return None


def get_task_status(task_id):
    """
    Requires beaker-client package installed and configured
    """
    if not stqe.host.restraint.is_restraint_job():
        return None

    if not task_id:
        _print("FAIL: get_task_status() - requires task ID")
        return None

    cmd = "bkr job-results --prettyxml T:%s" % task_id
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: get_task_status() - Could not get beaker task result for T:%s" % task_id)
        print(output)
        return None

    lines = output.split("\n")
    status_regex = re.compile(r"<task.*status=\"(\S+)\"")
    for line in lines:
        m = status_regex.match(line)
        if m:
            return m.group(1)
    return None


def console_log_check(error_mgs):
    """
    Checks for error messages on console log ("Call Trace and segfault")
    """
    error = 0
    console_log_file = "/root/console.log"
    prev_console_log_file = "/root/console.log.prev"
    new_console_log_file = "/root/console.log.new"

    if not is_beaker_job():
        print("WARN: skip console_log_check as it doesn't seem to be a beaker job")
        return True

    lab_controller = os.environ['LAB_CONTROLLER']
    recipe_id = os.environ['BEAKER_RECIPE_ID']

    # get current console log
    url = "http://%s:8000/recipes/%s/logs/console.log" % (lab_controller, recipe_id)

    if run("wget -q %s -O %s" % (url, new_console_log_file)) != 0:
        print("INFO: Could not get console log")
        # return sucess if could not get console.log
        return True

    # if there was previous console log, we just check the new part
    run("diff -N -n --unidirectional-new-file %s %s > %s" % (
        prev_console_log_file, new_console_log_file, console_log_file))

    # backup the current full console.log
    # next time we run the test we will compare just
    # what has been appended to console.log
    run("mv -f %s %s" % (new_console_log_file, prev_console_log_file))

    print("INFO: Checking for errors on %s" % console_log_file)
    for msg in error_mgs:
        _, output = run("cat %s | grep -i '%s'" % (console_log_file, msg), return_output=True)
        if output:
            print("INFO found %s on %s" % (msg, console_log_file))
            stqe.host.restraint.log_submit(console_log_file)
            error = + 1

    if error:
        return False

    print("PASS: No errors on %s have been found." % console_log_file)
    return True


def is_beaker_job():
    """
    Checks if it is beaker job
    """
    need_env = ["BEAKER", "LAB_CONTROLLER"]
    for var in need_env:
        if var not in os.environ:
            return False

    return True



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import time
import traceback
from libsan.host.cmdline import run
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def _mirror():
    global TestObj, vg_name

    #
    # -i|--stripes
    #
    TestObj.tok("lvcreate -L4M --thin %s/pool" % vg_name)
    check_lv_expected_value(TestObj, "[pool_tdata]", vg_name, {"stripes": "1"})

    # --mirror option creates the needed _rimage/data_X LVs
    lvm2_version = linux.package_version("lvm2")
    if not lvm2_version:
        TestObj.tfail("Could not query lvm2 version")
        return

    # Behavior changed on BZ1462712, since then LV must be active to run lvconvert
    # the .el7 should not really matter when comparing versions
    if lvm2_version >= "2.02.171-6.el7":
        TestObj.tok("lvchange -an %s/pool" % vg_name)
        TestObj.tnok("lvconvert --type raid1 --mirrors 3 --yes %s/pool_tdata" % vg_name)
        TestObj.tok("lvchange -ay %s/pool" % vg_name)
    else:
        TestObj.tnok("lvconvert --type raid1 --mirrors 3 --yes %s/pool_tdata" % vg_name)
        TestObj.tok("lvchange -an %s/pool" % vg_name)

    TestObj.tok("lvconvert --type raid1 --mirrors 3 --yes %s/pool_tdata" % vg_name)
    time.sleep(5)
    TestObj.tok("lvconvert --type raid1 -m 1 --yes %s/pool_tmeta" % vg_name)
    TestObj.tok("lvchange -ay %s/pool" % vg_name)

    check_lv_expected_value(TestObj, "[pool_tdata_rimage_0]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rimage_1]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rimage_2]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rimage_3]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rmeta_0]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rmeta_1]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rmeta_2]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tdata_rmeta_3]", vg_name, {"stripes": "1"})

    check_lv_expected_value(TestObj, "[pool_tmeta_rimage_0]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tmeta_rimage_1]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tmeta_rmeta_0]", vg_name, {"stripes": "1"})
    check_lv_expected_value(TestObj, "[pool_tmeta_rmeta_1]", vg_name, {"stripes": "1"})

    run("lvs -a %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Provisioning Mirror test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _mirror()

    return True


def _clean_up():
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()
